@@ -1,4 +1,4 @@
-/*	$OpenBSD: clientloop.h,v 1.4 2001/02/06 22:43:02 markus Exp $	*/
+/*	$OpenBSD: clientloop.h,v 1.7 2002/04/22 21:04:52 markus Exp $	*/
 
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
@@ -36,4 +36,5 @@
  */
 
 /* Client side main loop for the interactive session. */
-int     client_loop(int have_pty, int escape_char, int id);
+int	 client_loop(int, int, int);
+void	 client_global_request_reply(int type, u_int32_t seq, void *ctxt);
